@@ -285,6 +285,8 @@ class PolarsDataset(Dataset, PyFuncConvertibleDatasetMixin):
             path=path,
             feature_names=feature_names,
             predictions=self._predictions,
+            name=self.name,
+            digest=self.digest,
         )
 
 
