@@ -13,8 +13,11 @@ from mlflow.entities.assessment import (
 )
 from mlflow.entities.dataset import Dataset
 from mlflow.entities.dataset_input import DatasetInput
+from mlflow.entities.dataset_record import DatasetRecord
+from mlflow.entities.dataset_record_source import DatasetRecordSource, DatasetRecordSourceType
 from mlflow.entities.dataset_summary import _DatasetSummary
 from mlflow.entities.document import Document
+from mlflow.entities.entity_type import EntityAssociationType
 from mlflow.entities.experiment import Experiment
 from mlflow.entities.experiment_tag import ExperimentTag
 from mlflow.entities.file_info import FileInfo
@@ -36,6 +39,7 @@ from mlflow.entities.run_inputs import RunInputs
 from mlflow.entities.run_outputs import RunOutputs
 from mlflow.entities.run_status import RunStatus
 from mlflow.entities.run_tag import RunTag
+from mlflow.entities.scorer import ScorerVersion
 from mlflow.entities.source_type import SourceType
 from mlflow.entities.span import LiveSpan, NoOpSpan, Span, SpanType
 from mlflow.entities.span_event import SpanEvent
@@ -48,6 +52,7 @@ from mlflow.entities.trace_location import (
     MlflowExperimentLocation,
     TraceLocation,
     TraceLocationType,
+    UCSchemaLocation,
 )
 from mlflow.entities.trace_state import TraceState
 from mlflow.entities.view_type import ViewType
@@ -60,6 +65,7 @@ from mlflow.entities.webhook import (
 
 __all__ = [
     "Experiment",
+    "ExperimentTag",
     "FileInfo",
     "Metric",
     "Param",
@@ -69,7 +75,7 @@ __all__ = [
     "RunInfo",
     "RunStatus",
     "RunTag",
-    "ExperimentTag",
+    "ScorerVersion",
     "SourceType",
     "ViewType",
     "LifecycleStage",
@@ -91,6 +97,7 @@ __all__ = [
     "TraceLocationType",
     "MlflowExperimentLocation",
     "InferenceTableLocation",
+    "UCSchemaLocation",
     "TraceState",
     "SpanStatusCode",
     "_DatasetSummary",
@@ -107,8 +114,30 @@ __all__ = [
     "AssessmentSourceType",
     "Expectation",
     "Feedback",
+    "EvaluationDataset",
+    "DatasetRecord",
+    "DatasetRecordSource",
+    "DatasetRecordSourceType",
+    "EntityAssociationType",
     "Webhook",
     "WebhookEvent",
     "WebhookStatus",
     "WebhookTestResult",
 ]
+
+
+def __getattr__(name):
+    """Lazy loading for EvaluationDataset to avoid circular imports."""
+    if name == "EvaluationDataset":
+        try:
+            from mlflow.entities.evaluation_dataset import EvaluationDataset
+
+            return EvaluationDataset
+        except ImportError:
+            # EvaluationDataset requires mlflow.data which may not be available
+            # in minimal installations like mlflow-tracing
+            raise AttributeError(
+                "EvaluationDataset is not available. It requires the mlflow.data module "
+                "which is not included in this installation."
+            )
+    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
