@@ -10,9 +10,9 @@ class LifecycleStage:
     @classmethod
     def view_type_to_stages(cls, view_type=ViewType.ALL):
         stages = []
-        if view_type == ViewType.ACTIVE_ONLY or view_type == ViewType.ALL:
+        if view_type in (ViewType.ACTIVE_ONLY, ViewType.ALL):
             stages.append(cls.ACTIVE)
-        if view_type == ViewType.DELETED_ONLY or view_type == ViewType.ALL:
+        if view_type in (ViewType.DELETED_ONLY, ViewType.ALL):
             stages.append(cls.DELETED)
         return stages
 
@@ -23,7 +23,7 @@ class LifecycleStage:
     @classmethod
     def matches_view_type(cls, view_type, lifecycle_stage):
         if not cls.is_valid(lifecycle_stage):
-            raise MlflowException("Invalid lifecycle stage '%s'" % str(lifecycle_stage))
+            raise MlflowException(f"Invalid lifecycle stage '{lifecycle_stage}'")
 
         if view_type == ViewType.ALL:
             return True
@@ -32,4 +32,4 @@ class LifecycleStage:
         elif view_type == ViewType.DELETED_ONLY:
             return lifecycle_stage == LifecycleStage.DELETED
         else:
-            raise MlflowException("Invalid view type '%s'" % str(view_type))
+            raise MlflowException(f"Invalid view type '{view_type}'")
