@@ -1,27 +1,44 @@
+import base64
 import json
 import logging
-from dataclasses import asdict
 from functools import lru_cache
-from typing import Any, Optional, Union
+from typing import Any, Union
 
+from google.protobuf.json_format import ParseDict
+from google.protobuf.struct_pb2 import Value
+from opentelemetry.proto.trace.v1.trace_pb2 import Span as OTelProtoSpan
+from opentelemetry.proto.trace.v1.trace_pb2 import Status as OTelProtoStatus
+from opentelemetry.sdk.resources import Resource as _OTelResource
 from opentelemetry.sdk.trace import Event as OTelEvent
 from opentelemetry.sdk.trace import ReadableSpan as OTelReadableSpan
 from opentelemetry.trace import NonRecordingSpan, SpanContext, TraceFlags
 from opentelemetry.trace import Span as OTelSpan
+from opentelemetry.trace import Status as OTelStatus
+from opentelemetry.trace import StatusCode as OTelStatusCode
 
 import mlflow
 from mlflow.entities.span_event import SpanEvent
 from mlflow.entities.span_status import SpanStatus, SpanStatusCode
 from mlflow.exceptions import MlflowException
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
-from mlflow.tracing.constant import SpanAttributeKey
+from mlflow.protos.databricks_trace_server_pb2 import Span as ProtoSpan
+from mlflow.tracing.constant import TRACE_REQUEST_ID_PREFIX, SpanAttributeKey
 from mlflow.tracing.utils import (
     TraceJSONEncoder,
     build_otel_context,
     decode_id,
     encode_span_id,
     encode_trace_id,
+    generate_mlflow_trace_id_from_otel_trace_id,
+    generate_trace_id_v4_from_otel_trace_id,
+    parse_trace_id_v4,
 )
+from mlflow.tracing.utils.otlp import (
+    _decode_otel_proto_anyvalue,
+    _otel_proto_bytes_to_id,
+    _set_otel_proto_anyvalue,
+)
+from mlflow.tracing.utils.processor import apply_span_processors
 
 _logger = logging.getLogger(__name__)
 
@@ -41,11 +58,12 @@ class SpanType:
     PARSER = "PARSER"
     EMBEDDING = "EMBEDDING"
     RERANKER = "RERANKER"
+    MEMORY = "MEMORY"
     UNKNOWN = "UNKNOWN"
 
 
 def create_mlflow_span(
-    otel_span: Any, request_id: str, span_type: Optional[str] = None
+    otel_span: Any, trace_id: str, span_type: str | None = None
 ) -> Union["Span", "LiveSpan", "NoOpSpan"]:
     """
     Factory function to create a span object.
@@ -57,7 +75,7 @@ def create_mlflow_span(
         return NoOpSpan()
 
     if isinstance(otel_span, OTelSpan):
-        return LiveSpan(otel_span, request_id, span_type)
+        return LiveSpan(otel_span, trace_id, span_type)
 
     if isinstance(otel_span, OTelReadableSpan):
         return Span(otel_span)
@@ -94,14 +112,15 @@ class Span:
 
     @property
     @lru_cache(maxsize=1)
-    def request_id(self) -> str:
-        """
-        The request ID of the span, a unique identifier for the trace it belongs to.
-        Request ID is equivalent to the trace ID in OpenTelemetry, but generated
-        differently by the tracing backend.
-        """
+    def trace_id(self) -> str:
+        """The trace ID of the span, a unique identifier for the trace it belongs to."""
         return self.get_attribute(SpanAttributeKey.REQUEST_ID)
 
+    @property
+    def request_id(self) -> str:
+        """Deprecated. Use `trace_id` instead."""
+        return self.trace_id
+
     @property
     def span_id(self) -> str:
         """The ID of the span. This is only unique within a trace."""
@@ -118,12 +137,12 @@ class Span:
         return self._span._start_time
 
     @property
-    def end_time_ns(self) -> Optional[int]:
+    def end_time_ns(self) -> int | None:
         """The end time of the span in nanosecond."""
         return self._span._end_time
 
     @property
-    def parent_id(self) -> Optional[str]:
+    def parent_id(self) -> str | None:
         """The span ID of the parent span."""
         if self._span.parent is None:
             return None
@@ -153,7 +172,7 @@ class Span:
     def _trace_id(self) -> str:
         """
         The OpenTelemetry trace ID of the span. Note that this should not be exposed to
-        the user, instead, use request_id as an unique identifier for a trace.
+        the user, instead, use trace_id property as an unique identifier for a trace.
         """
         return encode_trace_id(self._span.context.trace_id)
 
@@ -188,11 +207,11 @@ class Span:
 
     def __repr__(self):
         return (
-            f"{type(self).__name__}(name={self.name!r}, request_id={self.request_id!r}, "
+            f"{type(self).__name__}(name={self.name!r}, trace_id={self.trace_id!r}, "
             f"span_id={self.span_id!r}, parent_id={self.parent_id!r})"
         )
 
-    def get_attribute(self, key: str) -> Optional[Any]:
+    def get_attribute(self, key: str) -> Any | None:
         """
         Get a single attribute value from the span.
 
@@ -204,31 +223,35 @@ class Span:
         """
         return self._attributes.get(key)
 
-    def to_dict(self):
-        # NB: OpenTelemetry Span has to_json() method, but it will write many fields that
-        #  we don't use e.g. links, kind, resource, trace_state, etc. So we manually
-        #  cherry-pick the fields we need here.
+    def to_dict(self) -> dict[str, Any]:
         return {
+            "trace_id": self.trace_id,
+            "span_id": self.span_id,
+            "parent_span_id": self.parent_id,
             "name": self.name,
-            "context": {
-                "span_id": self.span_id,
-                "trace_id": self._trace_id,
+            "start_time_unix_nano": self.start_time_ns,
+            "end_time_unix_nano": self.end_time_ns,
+            "events": [
+                {
+                    "name": event.name,
+                    "time_unix_nano": event.timestamp,
+                    "attributes": event.attributes,
+                }
+                for event in self.events
+            ],
+            "status": {
+                "code": self.status.status_code.value,
+                "message": self.status.description,
             },
-            "parent_id": self.parent_id,
-            "start_time": self.start_time_ns,
-            "end_time": self.end_time_ns,
-            "status_code": self.status.status_code.value,
-            "status_message": self.status.description,
-            "attributes": dict(self._span.attributes),
-            "events": [asdict(event) for event in self.events],
+            # save the dumped attributes so they can be loaded correctly when deserializing
+            "attributes": {k: self._span.attributes.get(k) for k in self.attributes.keys()},
         }
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> "Span":
-        """
-        Create a Span object from the given dictionary.
-        """
+        """Create a Span object from the given dictionary."""
         try:
+            # Try to deserialize the span using the v3 schema
             request_id = data.get("attributes", {}).get(SpanAttributeKey.REQUEST_ID)
             if not request_id:
                 raise MlflowException(
@@ -236,25 +259,53 @@ class Span:
                     INVALID_PARAMETER_VALUE,
                 )
 
-            trace_id = decode_id(data["context"]["trace_id"])
-            span_id = decode_id(data["context"]["span_id"])
-            parent_id = decode_id(data["parent_id"]) if data["parent_id"] else None
+            if Span._is_span_v2_schema(data):
+                return cls.from_dict_v2(data)
+
+            if _is_base64_encoded(data["trace_id"]):
+                otel_trace_id = _decode_id_from_byte(data["trace_id"])
+                span_id = _decode_id_from_byte(data["span_id"])
+                # Parent ID always exists in proto (empty string) even if the span is a root span.
+                parent_id = (
+                    _decode_id_from_byte(data["parent_span_id"]) if data["parent_span_id"] else None
+                )
+                status = SpanStatus(
+                    status_code=SpanStatusCode.from_proto_status_code(data["status"]["code"]),
+                    description=data["status"].get("message"),
+                )
+            else:
+                otel_trace_id = decode_id(
+                    parse_trace_id_v4(data["trace_id"])[1].removeprefix(TRACE_REQUEST_ID_PREFIX)
+                )
+                span_id = decode_id(data["span_id"])
+                parent_id = decode_id(data["parent_span_id"]) if data["parent_span_id"] else None
+                status = SpanStatus(
+                    status_code=SpanStatusCode(data["status"]["code"]),
+                    description=data["status"].get("message"),
+                )
+
+            end_time_ns = data.get("end_time_unix_nano")
+            end_time_ns = int(end_time_ns) if end_time_ns else None
 
             otel_span = OTelReadableSpan(
                 name=data["name"],
-                context=build_otel_context(trace_id, span_id),
-                parent=build_otel_context(trace_id, parent_id) if parent_id else None,
-                start_time=data["start_time"],
-                end_time=data["end_time"],
+                context=build_otel_context(otel_trace_id, span_id),
+                parent=build_otel_context(otel_trace_id, parent_id) if parent_id else None,
+                start_time=int(data["start_time_unix_nano"]),
+                end_time=end_time_ns,
                 attributes=data["attributes"],
-                status=SpanStatus(data["status_code"], data["status_message"]).to_otel_status(),
+                status=status.to_otel_status(),
+                # Setting an empty resource explicitly. Otherwise OTel create a new Resource by
+                # Resource.create(), which introduces a significant overhead in some environments.
+                # https://github.com/mlflow/mlflow/issues/15625
+                resource=_OTelResource.get_empty(),
                 events=[
                     OTelEvent(
                         name=event["name"],
-                        timestamp=event["timestamp"],
-                        attributes=event["attributes"],
+                        timestamp=int(event["time_unix_nano"]),
+                        attributes=event.get("attributes", {}),
                     )
-                    for event in data["events"]
+                    for event in data.get("events", [])
                 ],
             )
             return cls(otel_span)
@@ -264,6 +315,179 @@ class Span:
                 INVALID_PARAMETER_VALUE,
             ) from e
 
+    @staticmethod
+    def _is_span_v2_schema(data: dict[str, Any]) -> bool:
+        return "context" in data
+
+    @classmethod
+    def from_dict_v2(cls, data: dict[str, Any]) -> "Span":
+        """Create a Span object from the given dictionary in v2 schema."""
+        trace_id = decode_id(data["context"]["trace_id"])
+        span_id = decode_id(data["context"]["span_id"])
+        parent_id = decode_id(data["parent_id"]) if data["parent_id"] else None
+
+        otel_span = OTelReadableSpan(
+            name=data["name"],
+            context=build_otel_context(trace_id, span_id),
+            parent=build_otel_context(trace_id, parent_id) if parent_id else None,
+            start_time=data["start_time"],
+            end_time=data["end_time"],
+            attributes=data["attributes"],
+            status=SpanStatus(data["status_code"], data["status_message"]).to_otel_status(),
+            # Setting an empty resource explicitly. Otherwise OTel create a new Resource by
+            # Resource.create(), which introduces a significant overhead in some environments.
+            # https://github.com/mlflow/mlflow/issues/15625
+            resource=_OTelResource.get_empty(),
+            events=[
+                OTelEvent(
+                    name=event["name"],
+                    timestamp=event["timestamp"],
+                    attributes=event["attributes"],
+                )
+                for event in data["events"]
+            ],
+        )
+        return cls(otel_span)
+
+    def to_proto(self):
+        """Convert into OTLP compatible proto object to sent to the Databricks Trace Server."""
+        otel_status = self._span.status
+        status = ProtoSpan.Status(
+            code=otel_status.status_code.value,
+            message=otel_status.description,
+        )
+        parent = _encode_span_id_to_byte(self._span.parent.span_id) if self._span.parent else b""
+
+        # NB: This is a workaround that some DBX internal code pass float timestamp
+        start_time_unix_nano = int(self._span.start_time) if self._span.start_time else None
+        end_time_unix_nano = int(self._span.end_time) if self._span.end_time else None
+
+        return ProtoSpan(
+            trace_id=_encode_trace_id_to_byte(self._span.context.trace_id),
+            span_id=_encode_span_id_to_byte(self._span.context.span_id),
+            trace_state=self._span.context.trace_state or "",
+            parent_span_id=parent,
+            name=self.name,
+            start_time_unix_nano=start_time_unix_nano,
+            end_time_unix_nano=end_time_unix_nano,
+            events=[event.to_proto() for event in self.events],
+            status=status,
+            attributes={k: ParseDict(v, Value()) for k, v in self._span.attributes.items()},
+        )
+
+    @classmethod
+    def from_otel_proto(cls, otel_proto_span, location_id: str | None = None) -> "Span":
+        """
+        Create a Span from an OpenTelemetry protobuf span.
+        This is an internal method used for receiving spans via OTel protocol.
+        """
+        trace_id = _otel_proto_bytes_to_id(otel_proto_span.trace_id)
+        span_id = _otel_proto_bytes_to_id(otel_proto_span.span_id)
+        parent_id = None
+        if otel_proto_span.parent_span_id:
+            parent_id = _otel_proto_bytes_to_id(otel_proto_span.parent_span_id)
+
+        # Convert OTel proto status code directly to OTel SDK status
+        if otel_proto_span.status.code == OTelProtoStatus.STATUS_CODE_OK:
+            status_code = OTelStatusCode.OK
+        elif otel_proto_span.status.code == OTelProtoStatus.STATUS_CODE_ERROR:
+            status_code = OTelStatusCode.ERROR
+        else:
+            status_code = OTelStatusCode.UNSET
+
+        mlflow_trace_id = (
+            generate_trace_id_v4_from_otel_trace_id(trace_id, location_id)
+            if location_id
+            else generate_mlflow_trace_id_from_otel_trace_id(trace_id)
+        )
+        otel_span = OTelReadableSpan(
+            name=otel_proto_span.name,
+            context=build_otel_context(trace_id, span_id),
+            parent=build_otel_context(trace_id, parent_id) if parent_id else None,
+            start_time=otel_proto_span.start_time_unix_nano,
+            end_time=otel_proto_span.end_time_unix_nano,
+            attributes={
+                # Include the MLflow trace request ID only if it's not already present in attributes
+                SpanAttributeKey.REQUEST_ID: mlflow_trace_id,
+                **{
+                    attr.key: _decode_otel_proto_anyvalue(attr.value)
+                    for attr in otel_proto_span.attributes
+                },
+            },
+            status=OTelStatus(status_code, otel_proto_span.status.message or None),
+            events=[
+                OTelEvent(
+                    name=event.name,
+                    timestamp=event.time_unix_nano,
+                    attributes={
+                        attr.key: _decode_otel_proto_anyvalue(attr.value)
+                        for attr in event.attributes
+                    },
+                )
+                for event in otel_proto_span.events
+            ],
+            resource=_OTelResource.get_empty(),
+        )
+
+        return cls(otel_span)
+
+    def to_otel_proto(self) -> OTelProtoSpan:
+        """
+        Convert to OpenTelemetry protobuf span format for OTLP export.
+        This is an internal method used by the REST store for logging spans.
+
+        Returns:
+            An OpenTelemetry protobuf Span message.
+        """
+        otel_span = OTelProtoSpan()
+        otel_span.trace_id = bytes.fromhex(self._trace_id)
+        otel_span.span_id = bytes.fromhex(self.span_id)
+
+        otel_span.name = self.name
+        otel_span.start_time_unix_nano = self.start_time_ns
+        if self.end_time_ns:
+            otel_span.end_time_unix_nano = self.end_time_ns
+
+        if self.parent_id:
+            otel_span.parent_span_id = bytes.fromhex(self.parent_id)
+
+        otel_span.status.CopyFrom(self.status.to_otel_proto_status())
+
+        for key, value in self.attributes.items():
+            attr = otel_span.attributes.add()
+            attr.key = key
+            _set_otel_proto_anyvalue(attr.value, value)
+
+        for event in self.events:
+            otel_event = event.to_otel_proto()
+            otel_span.events.append(otel_event)
+
+        return otel_span
+
+
+def _encode_span_id_to_byte(span_id: int | None) -> bytes:
+    # https://github.com/open-telemetry/opentelemetry-python/blob/e01fa0c77a7be0af77d008a888c2b6a707b05c3d/exporter/opentelemetry-exporter-otlp-proto-common/src/opentelemetry/exporter/otlp/proto/common/_internal/__init__.py#L131
+    return span_id.to_bytes(length=8, byteorder="big", signed=False)
+
+
+def _encode_trace_id_to_byte(trace_id: int) -> bytes:
+    # https://github.com/open-telemetry/opentelemetry-python/blob/e01fa0c77a7be0af77d008a888c2b6a707b05c3d/exporter/opentelemetry-exporter-otlp-proto-common/src/opentelemetry/exporter/otlp/proto/common/_internal/__init__.py#L135
+    return trace_id.to_bytes(length=16, byteorder="big", signed=False)
+
+
+def _decode_id_from_byte(trace_or_span_id_b64: str) -> int:
+    # Decoding the base64 encoded trace or span ID to bytes and then converting it to int.
+    bytes = base64.b64decode(trace_or_span_id_b64)
+    return int.from_bytes(bytes, byteorder="big", signed=False)
+
+
+def _is_base64_encoded(trace_or_span_id: str) -> bool:
+    try:
+        base64.b64decode(trace_or_span_id, validate=True)
+        return True
+    except Exception:
+        return False
+
 
 class LiveSpan(Span):
     """
@@ -277,7 +501,7 @@ class LiveSpan(Span):
     def __init__(
         self,
         otel_span: OTelSpan,
-        request_id: str,
+        trace_id: str,
         span_type: str = SpanType.UNKNOWN,
     ):
         """
@@ -297,8 +521,14 @@ class LiveSpan(Span):
 
         self._span = otel_span
         self._attributes = _SpanAttributesRegistry(otel_span)
-        self._attributes.set(SpanAttributeKey.REQUEST_ID, request_id)
+        self._attributes.set(SpanAttributeKey.REQUEST_ID, trace_id)
         self._attributes.set(SpanAttributeKey.SPAN_TYPE, span_type)
+        # Track the original span name for deduplication purposes during span logging.
+        # Why: When traces contain multiple spans with identical names (e.g., multiple "LLM"
+        # or "query" spans), it's difficult for users to distinguish between them in the UI
+        # and logs. As spans are logged, we incrementally add numeric suffixes (_1, _2, etc.) to
+        # make each span uniquely identifiable within its trace
+        self._original_name = otel_span.name
 
     def set_span_type(self, span_type: str):
         """Set the type of the span."""
@@ -331,7 +561,7 @@ class LiveSpan(Span):
         """Set a single attribute to the span."""
         self._attributes.set(key, value)
 
-    def set_status(self, status: Union[SpanStatusCode, str]):
+    def set_status(self, status: SpanStatusCode | str):
         """
         Set the status of the span.
 
@@ -365,24 +595,80 @@ class LiveSpan(Span):
         """
         self._span.add_event(event.name, event.attributes, event.timestamp)
 
-    def end(self, end_time: Optional[int] = None):
+    def record_exception(self, exception: str | Exception):
         """
-        End the span. This is a thin wrapper around the OpenTelemetry's end method but just
-        to handle the status update.
+        Record an exception on the span, adding an exception event and setting span status to ERROR.
 
-        This method should not be called directly by the user, only by called via fluent APIs
-        context exit or by MlflowClient APIs.
+        Args:
+            exception: The exception to record. Can be an Exception instance or a string
+                describing the exception.
+        """
+        if isinstance(exception, Exception):
+            self.add_event(SpanEvent.from_exception(exception))
+        elif isinstance(exception, str):
+            self.add_event(SpanEvent.from_exception(Exception(exception)))
+        else:
+            raise MlflowException(
+                "The `exception` parameter must be an Exception instance or a string.",
+                INVALID_PARAMETER_VALUE,
+            )
+
+        self.set_status(
+            SpanStatus(
+                status_code=SpanStatusCode.ERROR,
+                description=f"{type(exception).__name__}: {exception}",
+            )
+        )
+
+    def end(
+        self,
+        outputs: Any | None = None,
+        attributes: dict[str, Any] | None = None,
+        status: SpanStatus | str | None = None,
+        end_time_ns: int | None = None,
+    ):
+        """
+        End the span.
+
+        outputs: Outputs to set on the span.
+        attributes: A dictionary of attributes to set on the span. If the span already has
+            attributes, the new attributes will be merged with the existing ones. If the same
+            key already exists, the new value will overwrite the old one.
+        status: The status of the span. This can be a
+            :py:class:`SpanStatus <mlflow.entities.SpanStatus>` object or a string
+            representing the status code defined in
+            :py:class:`SpanStatusCode <mlflow.entities.SpanStatusCode>`
+            e.g. ``"OK"``, ``"ERROR"``. The default status is OK.
+        end_time_ns: The end time of the span in nano seconds since the UNIX epoch.
+            If not provided, the current time will be used.
 
         :meta private:
         """
-        # NB: In OpenTelemetry, status code remains UNSET if not explicitly set
-        # by the user. However, there is not way to set the status when using
-        # @mlflow.trace decorator. Therefore, we just automatically set the status
-        # to OK if it is not ERROR.
-        if self.status.status_code != SpanStatusCode.ERROR:
-            self.set_status(SpanStatus(SpanStatusCode.OK))
+        try:
+            self.set_attributes(attributes or {})
+            if outputs is not None:
+                self.set_outputs(outputs)
+            if status is not None:
+                self.set_status(status)
 
-        self._span.end(end_time=end_time)
+            # NB: In OpenTelemetry, status code remains UNSET if not explicitly set
+            # by the user. However, there is not way to set the status when using
+            # @mlflow.trace decorator. Therefore, we just automatically set the status
+            # to OK if it is not ERROR.
+            if self.status.status_code != SpanStatusCode.ERROR:
+                self.set_status(SpanStatus(SpanStatusCode.OK))
+
+            # Apply span processors
+            apply_span_processors(self)
+
+            self._span.end(end_time=end_time_ns)
+
+        except Exception as e:
+            _logger.warning(
+                f"Failed to end span {self.span_id}: {e}. "
+                "For full traceback, set logging level to debug.",
+                exc_info=_logger.isEnabledFor(logging.DEBUG),
+            )
 
     def from_dict(cls, data: dict[str, Any]) -> "Span":
         raise NotImplementedError("The `from_dict` method is not supported for the LiveSpan class.")
@@ -400,9 +686,11 @@ class LiveSpan(Span):
     def from_immutable_span(
         cls,
         span: Span,
-        parent_span_id: Optional[str] = None,
-        request_id: Optional[str] = None,
-        trace_id: Optional[str] = None,
+        parent_span_id: str | None = None,
+        trace_id: str | None = None,
+        experiment_id: str | None = None,
+        otel_trace_id: str | None = None,
+        end_trace: bool = True,
     ) -> "LiveSpan":
         """
         Create a new LiveSpan object from the given immutable span by
@@ -410,17 +698,20 @@ class LiveSpan(Span):
 
         This is particularly useful when we merging a remote trace into the current trace.
         We cannot merge the remote trace directly, because it is already stored as an immutable
-        span, meaning that we cannot update metadata like request ID, trace ID, parent span ID,
+        span, meaning that we cannot update metadata like trace ID, parent span ID,
         which are necessary for merging the trace.
 
         Args:
             span: The immutable span object to clone.
             parent_span_id: The parent span ID of the new span.
                 If it is None, the span will be created as a root span.
-            request_id: The request ID to be set on the new span. Specify this if you want to
-                create the new span with a different request ID from the original span.
-            trace_id: The trace ID of the new span in hex encoded format. Specify this if you
-                want to create the new span with a different trace ID from the original span
+            trace_id: The trace ID to be set on the new span. Specify this if you want to
+                create the new span with a particular trace ID.
+            experiment_id: The experiment ID to be set on the new span. If not specified, the
+                experiment ID will be set to the current experiment ID.
+            otel_trace_id: The OpenTelemetry trace ID of the new span in hex encoded format.
+                If not specified, the newly generated trace ID will be used.
+            end_trace: Whether to end the trace after cloning the span. Default is True.
 
         Returns:
             The new LiveSpan object with the same state as the original span.
@@ -430,45 +721,50 @@ class LiveSpan(Span):
         from mlflow.tracing.trace_manager import InMemoryTraceManager
 
         trace_manager = InMemoryTraceManager.get_instance()
-        request_id = request_id or span.request_id
-        parent_span = trace_manager.get_span_from_id(request_id, parent_span_id)
+        parent_span = trace_manager.get_span_from_id(trace_id, parent_span_id)
 
         # Create a new span with the same name, parent, and start time
         otel_span = mlflow.tracing.provider.start_detached_span(
             name=span.name,
             parent=parent_span._span if parent_span else None,
             start_time_ns=span.start_time_ns,
+            experiment_id=experiment_id,
         )
-        # otel_span._span_processor = span._span._span_processor
-        clone_span = LiveSpan(otel_span, request_id, span.span_type)
+        # The latter one from attributes is the newly generated trace ID by the span processor.
+        trace_id = trace_id or json.loads(otel_span.attributes.get(SpanAttributeKey.REQUEST_ID))
+        clone_span = LiveSpan(otel_span, trace_id, span.span_type)
 
         # Copy all the attributes, inputs, outputs, and events from the original span
         clone_span.set_status(span.status)
         clone_span.set_attributes(
             {k: v for k, v in span.attributes.items() if k != SpanAttributeKey.REQUEST_ID}
         )
-        clone_span.set_inputs(span.inputs)
-        clone_span.set_outputs(span.outputs)
+        if span.inputs:
+            clone_span.set_inputs(span.inputs)
+        if span.outputs:
+            clone_span.set_outputs(span.outputs)
         for event in span.events:
             clone_span.add_event(event)
 
         # Update trace ID and span ID
         context = span._span.get_span_context()
         clone_span._span._context = SpanContext(
-            # Override trace_id if provided, otherwise use the original trace ID
-            trace_id=decode_id(trace_id) or context.trace_id,
+            # Override otel_trace_id if provided, otherwise use the new trace ID
+            trace_id=decode_id(otel_trace_id) if otel_trace_id else otel_span.context.trace_id,
+            # Re-use same span ID as their ID space is local to the trace
             span_id=context.span_id,
             is_remote=context.is_remote,
             # Override trace flag as if it is sampled within current context.
             trace_flags=TraceFlags(TraceFlags.SAMPLED),
         )
 
-        # Mark the span completed with the original end time
-        clone_span.end(end_time=span.end_time_ns)
+        if end_trace:
+            clone_span.end(end_time_ns=span.end_time_ns)
+
         return clone_span
 
 
-NO_OP_SPAN_REQUEST_ID = "MLFLOW_NO_OP_SPAN_REQUEST_ID"
+NO_OP_SPAN_TRACE_ID = "MLFLOW_NO_OP_SPAN_TRACE_ID"
 
 
 class NoOpSpan(Span):
@@ -495,11 +791,11 @@ class NoOpSpan(Span):
         self._attributes = {}
 
     @property
-    def request_id(self):
+    def trace_id(self):
         """
-        No-op span returns a special request ID to distinguish it from the real spans.
+        No-op span returns a special trace ID to distinguish it from the real spans.
         """
-        return NO_OP_SPAN_REQUEST_ID
+        return NO_OP_SPAN_TRACE_ID
 
     @property
     def span_id(self):
@@ -551,7 +847,16 @@ class NoOpSpan(Span):
     def add_event(self, event: SpanEvent):
         pass
 
-    def end(self):
+    def record_exception(self, exception: str | Exception):
+        pass
+
+    def end(
+        self,
+        outputs: Any | None = None,
+        attributes: dict[str, Any] | None = None,
+        status: SpanStatus | str | None = None,
+        end_time_ns: int | None = None,
+    ):
         pass
 
 
@@ -577,11 +882,10 @@ class _SpanAttributesRegistry:
         if serialized_value:
             try:
                 return json.loads(serialized_value)
-            except Exception as e:
-                _logger.warning(
-                    f"Failed to get value for key {key}, make sure you set the attribute "
-                    f"on mlflow Span class instead of directly to the OpenTelemetry span. {e}"
-                )
+            except Exception:
+                # If failed to deserialize (e.g., string value is directly set to OTel span),
+                # return the original value as is.
+                return serialized_value
 
     def set(self, key: str, value: Any):
         if not isinstance(key, str):
