@@ -11,8 +11,9 @@ from mlflow.entities.assessment import Assessment
 from mlflow.entities.trace_location import TraceLocation
 from mlflow.entities.trace_state import TraceState
 from mlflow.entities.trace_status import TraceStatus
+from mlflow.protos.databricks_tracing_pb2 import TraceInfo as ProtoTraceInfoV4
 from mlflow.protos.service_pb2 import TraceInfoV3 as ProtoTraceInfoV3
-from mlflow.tracing.constant import TRACE_SCHEMA_VERSION, TRACE_SCHEMA_VERSION_KEY, TraceMetadataKey
+from mlflow.tracing.constant import TraceMetadataKey
 
 
 @dataclass
@@ -54,18 +55,15 @@ class TraceInfo(_MlflowObject):
     tags: dict[str, str] = field(default_factory=dict)
     assessments: list[Assessment] = field(default_factory=list)
 
-    def __post_init__(self):
-        # NB: MLflow automatically converts trace metadata and spans to V3 format, even if the
-        # trace was originally created in V2 format with an earlier version of MLflow. Accordingly,
-        # we also update the `TRACE_SCHEMA_VERSION_KEY` in the trace metadata to V3 for consistency
-        self.trace_metadata[TRACE_SCHEMA_VERSION_KEY] = str(TRACE_SCHEMA_VERSION)
-
     def to_dict(self) -> dict[str, Any]:
         """Convert the TraceInfoV3 object to a dictionary."""
         res = MessageToDict(self.to_proto(), preserving_proto_field_name=True)
         if self.execution_duration is not None:
             res.pop("execution_duration", None)
             res["execution_duration_ms"] = self.execution_duration
+        # override trace_id to be the same as trace_info.trace_id since it's parsed
+        # when converting to proto
+        res["trace_id"] = self.trace_id
         return res
 
     @classmethod
@@ -96,9 +94,14 @@ class TraceInfo(_MlflowObject):
 
         return cls(**d)
 
-    def to_proto(self):
+    def to_proto(self) -> ProtoTraceInfoV3 | ProtoTraceInfoV4:
         from mlflow.entities.trace_info_v2 import _truncate_request_metadata, _truncate_tags
 
+        if self._is_v4():
+            from mlflow.utils.databricks_tracing_utils import trace_info_to_v4_proto
+
+            return trace_info_to_v4_proto(self)
+
         request_time = Timestamp()
         request_time.FromMilliseconds(self.request_time)
         execution_duration = None
@@ -127,14 +130,27 @@ class TraceInfo(_MlflowObject):
 
             return TraceInfoV2.from_proto(proto).to_v3()
 
+        # import inside the function to avoid introducing top-level dependency on
+        # mlflow.tracing.utils in entities module
+        from mlflow.tracing.utils import construct_trace_id_v4
+
+        trace_location = TraceLocation.from_proto(proto.trace_location)
+        if trace_location.uc_schema:
+            trace_id = construct_trace_id_v4(
+                location=f"{trace_location.uc_schema.catalog_name}.{trace_location.uc_schema.schema_name}",
+                trace_id=proto.trace_id,
+            )
+        else:
+            trace_id = proto.trace_id
+
         return cls(
-            trace_id=proto.trace_id,
+            trace_id=trace_id,
             client_request_id=(
                 proto.client_request_id if proto.HasField("client_request_id") else None
             ),
-            trace_location=TraceLocation.from_proto(proto.trace_location),
-            request_preview=proto.request_preview,
-            response_preview=proto.response_preview,
+            trace_location=trace_location,
+            request_preview=proto.request_preview if proto.HasField("request_preview") else None,
+            response_preview=proto.response_preview if proto.HasField("response_preview") else None,
             request_time=proto.request_time.ToMilliseconds(),
             execution_duration=(
                 proto.execution_duration.ToMilliseconds()
@@ -218,3 +234,6 @@ class TraceInfo(_MlflowObject):
         if usage_json := self.trace_metadata.get(TraceMetadataKey.TOKEN_USAGE):
             return json.loads(usage_json)
         return None
+
+    def _is_v4(self) -> bool:
+        return self.trace_location.uc_schema is not None
