@@ -1,5 +1,3 @@
-from typing import Optional
-
 from mlflow.gateway.base_models import RequestModel, ResponseModel
 from mlflow.types.chat import BaseRequestPayload
 from mlflow.utils import IS_PYDANTIC_V2_OR_NEWER
@@ -17,7 +15,7 @@ _REQUEST_PAYLOAD_EXTRA_SCHEMA = {
 
 class RequestPayload(BaseRequestPayload, RequestModel):
     prompt: str
-    model: Optional[str] = None
+    model: str | None = None
 
     class Config:
         if IS_PYDANTIC_V2_OR_NEWER:
@@ -29,13 +27,13 @@ class RequestPayload(BaseRequestPayload, RequestModel):
 class Choice(ResponseModel):
     index: int
     text: str
-    finish_reason: Optional[str] = None
+    finish_reason: str | None = None
 
 
 class CompletionsUsage(ResponseModel):
-    prompt_tokens: Optional[int] = None
-    completion_tokens: Optional[int] = None
-    total_tokens: Optional[int] = None
+    prompt_tokens: int | None = None
+    completion_tokens: int | None = None
+    total_tokens: int | None = None
 
 
 _RESPONSE_PAYLOAD_EXTRA_SCHEMA = {
@@ -53,7 +51,7 @@ _RESPONSE_PAYLOAD_EXTRA_SCHEMA = {
 
 
 class ResponsePayload(ResponseModel):
-    id: Optional[str] = None
+    id: str | None = None
     object: str = "text_completion"
     created: int
     model: str
@@ -68,14 +66,14 @@ class ResponsePayload(ResponseModel):
 
 
 class StreamDelta(ResponseModel):
-    role: Optional[str] = None
-    content: Optional[str] = None
+    role: str | None = None
+    content: str | None = None
 
 
 class StreamChoice(ResponseModel):
     index: int
-    finish_reason: Optional[str] = None
-    delta: StreamDelta
+    finish_reason: str | None = None
+    text: str | None = None
 
 
 _STREAM_RESPONSE_PAYLOAD_EXTRA_SCHEMA = {
@@ -96,7 +94,7 @@ _STREAM_RESPONSE_PAYLOAD_EXTRA_SCHEMA = {
 
 
 class StreamResponsePayload(ResponseModel):
-    id: Optional[str] = None
+    id: str | None = None
     object: str = "text_completion_chunk"
     created: int
     model: str
