@@ -7,7 +7,7 @@ import posixpath
 import re
 import textwrap
 import warnings
-from typing import Any, AsyncGenerator, Optional
+from typing import Any, AsyncGenerator
 from urllib.parse import urlparse
 
 from mlflow.environment_variables import MLFLOW_GATEWAY_URI
@@ -16,7 +16,7 @@ from mlflow.gateway.constants import MLFLOW_AI_GATEWAY_MOSAICML_CHAT_SUPPORTED_M
 from mlflow.utils.uri import append_to_uri_path
 
 _logger = logging.getLogger(__name__)
-_gateway_uri: Optional[str] = None
+_gateway_uri: str | None = None
 
 
 def is_valid_endpoint_name(name: str) -> bool:
@@ -30,7 +30,12 @@ def is_valid_endpoint_name(name: str) -> bool:
 
 
 def check_configuration_route_name_collisions(config):
-    routes = config.get("routes") or config.get("endpoints") or []
+    if "routes" in config:
+        raise MlflowException.invalid_parameter_value(
+            "The 'routes' field is not supported in the configuration file, "
+            "use 'endpoints' instead."
+        )
+    routes = config.get("endpoints") or []
     if len(routes) < 2:
         return
     names = [route["name"] for route in routes]
@@ -42,24 +47,13 @@ def check_configuration_route_name_collisions(config):
 
 
 def check_configuration_deprecated_fields(config):
-    if "routes" in config:
-        warnings.warn(
-            "The 'routes' configuration key has been deprecated and will be removed in an"
-            " upcoming release. Use 'endpoints' instead.",
-            FutureWarning,
-            stacklevel=2,
-        )
-
-    routes = config.get("routes", []) or config.get("endpoints", [])
+    routes = config.get("endpoints", [])
     for route in routes:
         if "route_type" in route:
-            warnings.warn(
-                "The 'route_type' configuration key has been deprecated and will be removed in an"
-                " upcoming release. Use 'endpoint_type' instead.",
-                FutureWarning,
-                stacklevel=2,
+            raise MlflowException.invalid_parameter_value(
+                "The 'route_type' configuration key is not supported in the configuration file. "
+                "Use 'endpoint_type' instead."
             )
-            break
 
 
 def kill_child_processes(parent_pid):
@@ -100,7 +94,7 @@ def _get_indent(s: str) -> str:
     return ""
 
 
-def _prepend(docstring: Optional[str], text: str) -> str:
+def _prepend(docstring: str | None, text: str) -> str:
     if not docstring:
         return text
 
