@@ -8,12 +8,9 @@ The API docs can be found here:
 
 from typing import Any
 
+from mlflow.genai.labeling.databricks_utils import get_databricks_review_app
 from mlflow.genai.labeling.labeling import Agent, LabelingSession, ReviewApp
-
-_ERROR_MSG = (
-    "The `databricks-agents` package is required to use `mlflow.genai.labeling`. "
-    "Please install it with `pip install databricks-agents`."
-)
+from mlflow.genai.labeling.stores import _get_labeling_store
 
 
 def get_review_app(experiment_id: str | None = None) -> "ReviewApp":
@@ -30,12 +27,7 @@ def get_review_app(experiment_id: str | None = None) -> "ReviewApp":
     Returns:
         ReviewApp: The review app.
     """
-    try:
-        from databricks.agents.review_app import get_review_app as _get_review_app
-    except ImportError:
-        raise ImportError(_ERROR_MSG) from None
-
-    return ReviewApp(_get_review_app(experiment_id))
+    return ReviewApp(get_databricks_review_app(experiment_id))
 
 
 def create_labeling_session(
@@ -64,20 +56,14 @@ def create_labeling_session(
     Returns:
         LabelingSession: The created labeling session.
     """
-    try:
-        from databricks.agents.review_app import get_review_app as _get_review_app
-    except ImportError:
-        raise ImportError(_ERROR_MSG) from None
-
-    return LabelingSession(
-        _get_review_app().create_labeling_session(
-            name=name,
-            assigned_users=assigned_users or [],
-            agent=agent,
-            label_schemas=label_schemas or [],
-            enable_multi_turn_chat=enable_multi_turn_chat,
-            custom_inputs=custom_inputs,
-        )
+    store = _get_labeling_store()
+    return store.create_labeling_session(
+        name=name,
+        assigned_users=assigned_users,
+        agent=agent,
+        label_schemas=label_schemas,
+        enable_multi_turn_chat=enable_multi_turn_chat,
+        custom_inputs=custom_inputs,
     )
 
 
@@ -91,12 +77,8 @@ def get_labeling_sessions() -> list[LabelingSession]:
     Returns:
         list[LabelingSession]: The list of labeling sessions.
     """
-    try:
-        from databricks.agents.review_app import get_review_app as _get_review_app
-    except ImportError:
-        raise ImportError(_ERROR_MSG) from None
-
-    return [LabelingSession(session) for session in _get_review_app().get_labeling_sessions()]
+    store = _get_labeling_store()
+    return store.get_labeling_sessions()
 
 
 def get_labeling_session(run_id: str) -> LabelingSession:
@@ -112,21 +94,11 @@ def get_labeling_session(run_id: str) -> LabelingSession:
     Returns:
         LabelingSession: The labeling session.
     """
-    labeling_sessions = get_labeling_sessions()
-    labeling_session = next(
-        (
-            labeling_session
-            for labeling_session in labeling_sessions
-            if labeling_session.mlflow_run_id == run_id
-        ),
-        None,
-    )
-    if labeling_session is None:
-        raise ValueError(f"Labeling session with run_id `{run_id}` not found")
-    return LabelingSession(labeling_session)
+    store = _get_labeling_store()
+    return store.get_labeling_session(run_id)
 
 
-def delete_labeling_session(labeling_session: LabelingSession) -> "ReviewApp":
+def delete_labeling_session(labeling_session: LabelingSession):
     """Delete a labeling session from the review app.
 
     .. note::
@@ -135,16 +107,18 @@ def delete_labeling_session(labeling_session: LabelingSession) -> "ReviewApp":
 
     Args:
         labeling_session: The labeling session to delete.
-
-    Returns:
-        ReviewApp: The review app.
     """
-    try:
-        from databricks.agents.review_app import get_review_app as _get_review_app
-    except ImportError:
-        raise ImportError(_ERROR_MSG) from None
+    store = _get_labeling_store()
+    store.delete_labeling_session(labeling_session)
 
-    return ReviewApp(_get_review_app().delete_labeling_session(labeling_session._session))
+    # For backwards compatibility, return a ReviewApp instance only if using Databricks store
+    from mlflow.genai.labeling.stores import DatabricksLabelingStore
+
+    if isinstance(store, DatabricksLabelingStore):
+        return ReviewApp(get_databricks_review_app())
+    else:
+        # For non-Databricks stores, we can't return a meaningful ReviewApp
+        return None
 
 
 __all__ = [
