@@ -39,7 +39,6 @@ except ImportError as e:
 
 from mlflow.langchain.utils.chat import convert_lc_message_to_chat_message
 from mlflow.types.agent import ChatAgentMessage
-from mlflow.utils.annotations import experimental
 
 
 def _add_agent_messages(
@@ -74,7 +73,6 @@ def _add_agent_messages(
     return merged
 
 
-@experimental(version="2.21.0")
 class ChatAgentState(TypedDict):
     """
     Helper class that enables building a LangGraph agent that produces ChatAgent-compatible
@@ -293,7 +291,6 @@ def parse_message(
     return chat_agent_msg.model_dump_compat(exclude_none=True)
 
 
-@experimental(version="2.21.0")
 class ChatAgentToolNode(ToolNode):
     """
     Helper class to make ToolNodes be compatible with
