@@ -11,58 +11,64 @@ Paddle (native) format
     since `predict()` is required for pyfunc model inference.
 """
 
-import os
 import logging
+import os
+from typing import Any
+
 import yaml
 
 import mlflow
 from mlflow import pyfunc
-from mlflow.models import Model
+from mlflow.models import Model, ModelInputExample, ModelSignature
 from mlflow.models.model import MLMODEL_FILE_NAME
-from mlflow.models.signature import ModelSignature
-from mlflow.models.utils import ModelInputExample, _save_example
+from mlflow.models.signature import _infer_signature_from_input_example
+from mlflow.models.utils import _save_example
+from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
+from mlflow.utils.autologging_utils import autologging_integration, safe_patch
+from mlflow.utils.docstring_utils import LOG_MODEL_PARAM_DOCS, format_docstring
 from mlflow.utils.environment import (
-    _mlflow_conda_env,
-    _validate_env_arguments,
-    _process_pip_requirements,
-    _process_conda_env,
     _CONDA_ENV_FILE_NAME,
-    _REQUIREMENTS_FILE_NAME,
     _CONSTRAINTS_FILE_NAME,
     _PYTHON_ENV_FILE_NAME,
+    _REQUIREMENTS_FILE_NAME,
+    _mlflow_conda_env,
+    _process_conda_env,
+    _process_pip_requirements,
     _PythonEnv,
+    _validate_env_arguments,
 )
-from mlflow.utils.requirements_utils import _get_pinned_requirement
-from mlflow.utils.docstring_utils import format_docstring, LOG_MODEL_PARAM_DOCS
 from mlflow.utils.file_utils import write_to
 from mlflow.utils.model_utils import (
+    _add_code_from_conf_to_system_path,
     _get_flavor_configuration,
     _validate_and_copy_code_paths,
-    _add_code_from_conf_to_system_path,
     _validate_and_prepare_target_save_path,
 )
-from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
-from mlflow.utils.autologging_utils import autologging_integration, safe_patch
+from mlflow.utils.requirements_utils import _get_pinned_requirement
 
 FLAVOR_NAME = "paddle"
 
+_MODEL_DATA_SUBPATH = "model"
+
 _logger = logging.getLogger(__name__)
 
 
 def get_default_pip_requirements():
     """
-    :return: A list of default pip requirements for MLflow Models produced by this flavor.
-             Calls to :func:`save_model()` and :func:`log_model()` produce a pip environment
-             that, at minimum, contains these requirements.
+    Returns:
+        A list of default pip requirements for MLflow Models produced by this flavor.
+        Calls to :func:`save_model()` and :func:`log_model()` produce a pip environment
+        that, at minimum, contains these requirements.
     """
     return [_get_pinned_requirement("paddlepaddle", module="paddle")]
 
 
 def get_default_conda_env():
     """
-    :return: The default Conda environment for MLflow Models produced by calls to
-             :func:`save_model()` and :func:`log_model()`.
+    Returns:
+        The default Conda environment for MLflow Models produced by calls to
+        :func:`save_model()` and :func:`log_model()`.
     """
     return _mlflow_conda_env(additional_pip_deps=get_default_pip_requirements())
 
@@ -89,40 +95,20 @@ def save_model(
         - :py:mod:`mlflow.pyfunc`. NOTE: This flavor is only included for paddle models
           that define `predict()`, since `predict()` is required for pyfunc model inference.
 
-    :param pd_model: paddle model to be saved.
-    :param path: Local path where the model is to be saved.
-    :param training: Only valid when saving a model trained using the PaddlePaddle high level API.
-                     If set to True, the saved model supports both re-training and
-                     inference. If set to False, it only supports inference.
-    :param conda_env: {{ conda_env }}
-    :param code_paths: A list of local filesystem paths to Python file dependencies (or directories
-                       containing file dependencies). These files are *prepended* to the system
-                       path when the model is loaded.
-    :param mlflow_model: :py:mod:`mlflow.models.Model` this flavor is being added to.
-    :param signature: :py:class:`ModelSignature <mlflow.models.ModelSignature>`
-                      describes model input and output :py:class:`Schema <mlflow.types.Schema>`.
-                      The model signature can be :py:func:`inferred <mlflow.models.infer_signature>`
-                      from datasets with valid model input (e.g. the training dataset with target
-                      column omitted) and valid model output (e.g. model predictions generated on
-                      the training dataset), for example:
-                      .. code-block:: python
-
-                        from mlflow.models.signature import infer_signature
-
-                        train = df.drop_column("target_label")
-                        predictions = ...  # compute model predictions
-                        signature = infer_signature(train, predictions)
-    :param input_example: Input example provides one or several instances of valid
-                          model input. The example can be used as a hint of what data to feed the
-                          model. The given example will be converted to a Pandas DataFrame and then
-                          serialized to json using the Pandas split-oriented format. Bytes are
-                          base64-encoded.
-    :param pip_requirements: {{ pip_requirements }}
-    :param extra_pip_requirements: {{ extra_pip_requirements }}
-    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
-
-                     .. Note:: Experimental: This parameter may change or be removed in a future
-                                             release without warning.
+    Args:
+        pd_model: paddle model to be saved.
+        path: Local path where the model is to be saved.
+        training: Only valid when saving a model trained using the PaddlePaddle high level API.
+            If set to True, the saved model supports both re-training and
+            inference. If set to False, it only supports inference.
+        conda_env: {{ conda_env }}
+        code_paths: {{ code_paths }}
+        mlflow_model: :py:mod:`mlflow.models.Model` this flavor is being added to.
+        signature: {{ signature }}
+        input_example: {{ input_example }}
+        pip_requirements: {{ pip_requirements }}
+        extra_pip_requirements: {{ extra_pip_requirements }}
+        metadata: {{ metadata }}
 
     .. code-block:: python
         :caption: Example
@@ -142,15 +128,12 @@ def save_model(
         def load_data():
             # dataset on boston housing prediction
             X, y = load_diabetes(return_X_y=True, as_frame=True)
-
             min_max_scaler = preprocessing.MinMaxScaler()
             X_min_max = min_max_scaler.fit_transform(X)
             X_normalized = preprocessing.scale(X_min_max, with_std=False)
-
             X_train, X_test, y_train, y_test = train_test_split(
                 X_normalized, y, test_size=0.2, random_state=42
             )
-
             y_train = y_train.reshape(-1, 1)
             y_test = y_test.reshape(-1, 1)
             return np.concatenate((X_train, y_train), axis=1), np.concatenate(
@@ -161,7 +144,6 @@ def save_model(
         class Regressor(paddle.nn.Layer):
             def __init__(self):
                 super().__init__()
-
                 self.fc = Linear(in_features=13, out_features=1)
 
             @paddle.jit.to_static
@@ -174,10 +156,8 @@ def save_model(
         model.train()
         training_data, test_data = load_data()
         opt = paddle.optimizer.SGD(learning_rate=0.01, parameters=model.parameters())
-
         EPOCH_NUM = 10
         BATCH_SIZE = 10
-
         for epoch_id in range(EPOCH_NUM):
             np.random.shuffle(training_data)
             mini_batches = [
@@ -189,27 +169,19 @@ def save_model(
                 y = np.array(mini_batch[:, -1:]).astype("float32")
                 house_features = paddle.to_tensor(x)
                 prices = paddle.to_tensor(y)
-
                 predicts = model(house_features)
-
                 loss = F.square_error_cost(predicts, label=prices)
                 avg_loss = paddle.mean(loss)
                 if iter_id % 20 == 0:
-                    print(
-                        "epoch: {}, iter: {}, loss is: {}".format(
-                            epoch_id, iter_id, avg_loss.numpy()
-                        )
-                    )
-
+                    print(f"epoch: {epoch_id}, iter: {iter_id}, loss is: {avg_loss.numpy()}")
                 avg_loss.backward()
                 opt.step()
                 opt.clear_grad()
-
         mlflow.log_param("learning_rate", 0.01)
-        mlflow.paddle.log_model(model, "model")
+        mlflow.paddle.log_model(model, name="model")
         sk_path_dir = "./test-out"
         mlflow.paddle.save_model(model, sk_path_dir)
-        print("Model saved in run %s" % mlflow.active_run().info.run_uuid)
+        print("Model saved in run %s" % mlflow.active_run().info.run_id)
     """
     import paddle
 
@@ -220,14 +192,20 @@ def save_model(
 
     if mlflow_model is None:
         mlflow_model = Model()
+    saved_example = _save_example(mlflow_model, input_example, path)
+
+    if signature is None and saved_example is not None:
+        wrapped_model = _PaddleWrapper(pd_model)
+        signature = _infer_signature_from_input_example(saved_example, wrapped_model)
+    elif signature is False:
+        signature = None
+
     if signature is not None:
         mlflow_model.signature = signature
-    if input_example is not None:
-        _save_example(mlflow_model, input_example, path)
     if metadata is not None:
         mlflow_model.metadata = metadata
 
-    model_data_subpath = "model"
+    model_data_subpath = _MODEL_DATA_SUBPATH
     output_path = os.path.join(path, model_data_subpath)
 
     if isinstance(pd_model, paddle.Model):
@@ -289,27 +267,28 @@ def save_model(
 def load_model(model_uri, model=None, dst_path=None, **kwargs):
     """
     Load a paddle model from a local file or a run.
-    :param model_uri: The location, in URI format, of the MLflow model, for example:
 
+    Args:
+        model_uri: The location, in URI format, of the MLflow model, for example:
             - ``/Users/me/path/to/local/model``
             - ``relative/path/to/local/model``
             - ``s3://my_bucket/path/to/model``
             - ``runs:/<mlflow_run_id>/run-relative/path/to/model``
             - ``models:/<model_name>/<model_version>``
             - ``models:/<model_name>/<stage>``
-
-    :param model: Required when loading a `paddle.Model` model saved with `training=True`.
-    :param dst_path: The local filesystem path to which to download the model artifact.
-                     This directory must already exist. If unspecified, a local output
-                     path will be created.
-    :param kwargs: The keyword arguments to pass to `paddle.jit.load`
-                   or `model.load`.
+        model: Required when loading a `paddle.Model` model saved with `training=True`.
+        dst_path: The local filesystem path to which to download the model artifact.
+            This directory must already exist. If unspecified, a local output
+            path will be created.
+        kwargs: The keyword arguments to pass to `paddle.jit.load`
+            or `model.load`.
 
     For more information about supported URI schemes, see
     `Referencing Artifacts <https://www.mlflow.org/docs/latest/concepts.html#
     artifact-locations>`_.
 
-    :return: A paddle model.
+    Returns:
+        A paddle model.
 
     .. code-block:: python
         :caption: Example
@@ -330,9 +309,7 @@ def load_model(model_uri, model=None, dst_path=None, **kwargs):
     if model is None:
         return paddle.jit.load(pd_model_artifacts_path, **kwargs)
     elif not isinstance(model, paddle.Model):
-        raise TypeError(
-            "Invalid object type `{}` for `model`, must be `paddle.Model`".format(type(model))
-        )
+        raise TypeError(f"Invalid object type `{type(model)}` for `model`, must be `paddle.Model`")
     else:
         contains_pdparams = _contains_pdparams(local_model_path)
         if not contains_pdparams:
@@ -349,7 +326,7 @@ def load_model(model_uri, model=None, dst_path=None, **kwargs):
 @format_docstring(LOG_MODEL_PARAM_DOCS.format(package_name=FLAVOR_NAME))
 def log_model(
     pd_model,
-    artifact_path,
+    artifact_path: str | None = None,
     training=False,
     conda_env=None,
     code_paths=None,
@@ -360,6 +337,12 @@ def log_model(
     pip_requirements=None,
     extra_pip_requirements=None,
     metadata=None,
+    name: str | None = None,
+    params: dict[str, Any] | None = None,
+    tags: dict[str, Any] | None = None,
+    model_type: str | None = None,
+    step: int = 0,
+    model_id: str | None = None,
 ):
     """
     Log a paddle model as an MLflow artifact for the current run. Produces an MLflow Model
@@ -369,47 +352,35 @@ def log_model(
         - :py:mod:`mlflow.pyfunc`. NOTE: This flavor is only included for paddle models
           that define `predict()`, since `predict()` is required for pyfunc model inference.
 
-    :param pd_model: paddle model to be saved.
-    :param artifact_path: Run-relative artifact path.
-    :param training: Only valid when saving a model trained using the PaddlePaddle high level API.
-                     If set to True, the saved model supports both re-training and
-                     inference. If set to False, it only supports inference.
-    :param conda_env: {{ conda_env }}
-    :param code_paths: A list of local filesystem paths to Python file dependencies (or directories
-                       containing file dependencies). These files are *prepended* to the system
-                       path when the model is loaded.
-    :param registered_model_name: If given, create a model version under
-                                  ``registered_model_name``, also creating a registered model if one
-                                  with the given name does not exist.
-    :param signature: :py:class:`ModelSignature <mlflow.models.ModelSignature>`
-                      describes model input and output :py:class:`Schema <mlflow.types.Schema>`.
-                      The model signature can be :py:func:`inferred <mlflow.models.infer_signature>`
-                      from datasets with valid model input (e.g. the training dataset with target
-                      column omitted) and valid model output (e.g. model predictions generated on
-                      the training dataset), for example:
-                      .. code-block:: python
+    Args:
+        pd_model: paddle model to be saved.
+        artifact_path: Deprecated. Use `name` instead.
+        training: Only valid when saving a model trained using the PaddlePaddle high level API.
+            If set to True, the saved model supports both re-training and
+            inference. If set to False, it only supports inference.
+        conda_env: {{ conda_env }}
+        code_paths: {{ code_paths }}
+        registered_model_name: If given, create a model version under
+            ``registered_model_name``, also creating a registered model if one
+            with the given name does not exist.
+        signature: {{ signature }}
+        input_example: {{ input_example }}
+        await_registration_for: Number of seconds to wait for the model version to finish
+            being created and is in ``READY`` status. By default, the function
+            waits for five minutes. Specify 0 or None to skip waiting.
+        pip_requirements: {{ pip_requirements }}
+        extra_pip_requirements: {{ extra_pip_requirements }}
+        metadata: {{ metadata }}
+        name: {{ name }}
+        params: {{ params }}
+        tags: {{ tags }}
+        model_type: {{ model_type }}
+        step: {{ step }}
+        model_id: {{ model_id }}
 
-                        from mlflow.models.signature import infer_signature
-
-                        train = df.drop_column("target_label")
-                        predictions = ...  # compute model predictions
-                        signature = infer_signature(train, predictions)
-    :param input_example: Input example provides one or several instances of valid
-                          model input. The example can be used as a hint of what data to feed the
-                          model. The given example will be converted to a Pandas DataFrame and then
-                          serialized to json using the Pandas split-oriented format. Bytes are
-                          base64-encoded.
-    :param await_registration_for: Number of seconds to wait for the model version to finish
-                            being created and is in ``READY`` status. By default, the function
-                            waits for five minutes. Specify 0 or None to skip waiting.
-    :param pip_requirements: {{ pip_requirements }}
-    :param extra_pip_requirements: {{ extra_pip_requirements }}
-    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
-
-                     .. Note:: Experimental: This parameter may change or be removed in a future
-                                             release without warning.
-    :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
-             metadata of the logged model.
+    Returns:
+        A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
+        metadata of the logged model.
 
     .. code-block:: python
         :caption: Example
@@ -417,32 +388,28 @@ def log_model(
         import mlflow.paddle
 
 
-        def load_data():
-            ...
+        def load_data(): ...
 
 
-        class Regressor:
-            ...
+        class Regressor: ...
 
 
         model = Regressor()
         model.train()
         training_data, test_data = load_data()
         opt = paddle.optimizer.SGD(learning_rate=0.01, parameters=model.parameters())
-
         EPOCH_NUM = 10
         BATCH_SIZE = 10
-
         for epoch_id in range(EPOCH_NUM):
             ...
-
         mlflow.log_param("learning_rate", 0.01)
-        mlflow.paddle.log_model(model, "model")
+        mlflow.paddle.log_model(model, name="model")
         sk_path_dir = ...
         mlflow.paddle.save_model(model, sk_path_dir)
     """
     return Model.log(
         artifact_path=artifact_path,
+        name=name,
         flavor=mlflow.paddle,
         pd_model=pd_model,
         conda_env=conda_env,
@@ -455,13 +422,20 @@ def log_model(
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
         metadata=metadata,
+        params=params,
+        tags=tags,
+        model_type=model_type,
+        step=step,
+        model_id=model_id,
     )
 
 
 def _load_pyfunc(path):
     """
-    Load PyFunc implementation. Called by ``pyfunc.load_model``.
-    :param path: Local filesystem path to the MLflow Model with the ``paddle`` flavor.
+    Loads PyFunc implementation. Called by ``pyfunc.load_model``.
+
+    Args:
+        path: Local filesystem path to the MLflow Model with the ``paddle`` flavor.
     """
     return _PaddleWrapper(load_model(path))
 
@@ -475,9 +449,28 @@ class _PaddleWrapper:
     def __init__(self, pd_model):
         self.pd_model = pd_model
 
-    def predict(self, data):
-        import pandas as pd
+    def get_raw_model(self):
+        """
+        Returns the underlying model.
+        """
+        return self.pd_model
+
+    def predict(
+        self,
+        data,
+        params: dict[str, Any] | None = None,
+    ):
+        """
+        Args:
+            data: Model input data.
+            params: Additional parameters to pass to the model for inference.
+
+        Returns:
+            Model predictions.
+        """
         import numpy as np
+        import paddle
+        import pandas as pd
 
         if isinstance(data, pd.DataFrame):
             inp_data = data.values.astype(np.float32)
@@ -494,7 +487,7 @@ class _PaddleWrapper:
 
         self.pd_model.eval()
 
-        predicted = self.pd_model(inp_data)
+        predicted = self.pd_model(paddle.to_tensor(inp_data))
         return pd.DataFrame(predicted.numpy())
 
 
@@ -511,7 +504,8 @@ def autolog(
     exclusive=False,
     silent=False,
     registered_model_name=None,
-):  # pylint: disable=unused-argument
+    extra_tags=None,
+):
     """
     Enables (or disables) and configures autologging from PaddlePaddle to MLflow.
 
@@ -520,21 +514,23 @@ def autolog(
     .. _paddle.Model:
         https://www.paddlepaddle.org.cn/documentation/docs/en/api/paddle/Model_en.html
 
-    :param log_every_n_epoch: If specified, logs metrics once every `n` epochs. By default, metrics
-                       are logged after every epoch.
-    :param log_models: If ``True``, trained models are logged as MLflow model artifacts.
-                       If ``False``, trained models are not logged.
-    :param disable: If ``True``, disables the PaddlePaddle autologging integration.
-                    If ``False``, enables the PaddlePaddle autologging integration.
-    :param exclusive: If ``True``, autologged content is not logged to user-created fluent runs.
-                      If ``False``, autologged content is logged to the active fluent run,
-                      which may be user-created.
-    :param silent: If ``True``, suppress all event logs and warnings from MLflow during PyTorch
-                   Lightning autologging. If ``False``, show all events and warnings during
-                   PaddlePaddle autologging.
-    :param registered_model_name: If given, each time a model is trained, it is registered as a
-                                  new model version of the registered model with this name.
-                                  The registered model is created if it does not already exist.
+    Args:
+        log_every_n_epoch: If specified, logs metrics once every `n` epochs. By default, metrics
+            are logged after every epoch.
+        log_models: If ``True``, trained models are logged as MLflow model artifacts.
+            If ``False``, trained models are not logged.
+        disable: If ``True``, disables the PaddlePaddle autologging integration.
+            If ``False``, enables the PaddlePaddle autologging integration.
+        exclusive: If ``True``, autologged content is not logged to user-created fluent runs.
+            If ``False``, autologged content is logged to the active fluent run,
+            which may be user-created.
+        silent: If ``True``, suppress all event logs and warnings from MLflow during PyTorch
+            Lightning autologging. If ``False``, show all events and warnings during
+            PaddlePaddle autologging.
+        registered_model_name: If given, each time a model is trained, it is registered as a
+            new model version of the registered model with this name.
+            The registered model is created if it does not already exist.
+        extra_tags: A dictionary of extra tags to set on each managed run created by autologging.
 
     .. code-block:: python
         :caption: Example
@@ -546,11 +542,11 @@ def autolog(
 
         def show_run_data(run_id):
             run = mlflow.get_run(run_id)
-            print("params: {}".format(run.data.params))
-            print("metrics: {}".format(run.data.metrics))
+            print(f"params: {run.data.params}")
+            print(f"metrics: {run.data.metrics}")
             client = MlflowClient()
             artifacts = [f.path for f in client.list_artifacts(run.info.run_id, "model")]
-            print("artifacts: {}".format(artifacts))
+            print(f"artifacts: {artifacts}")
 
 
         class LinearRegression(paddle.nn.Layer):
@@ -564,16 +560,12 @@ def autolog(
 
         train_dataset = paddle.text.datasets.UCIHousing(mode="train")
         eval_dataset = paddle.text.datasets.UCIHousing(mode="test")
-
         model = paddle.Model(LinearRegression())
         optim = paddle.optimizer.SGD(learning_rate=1e-2, parameters=model.parameters())
         model.prepare(optim, paddle.nn.MSELoss(), paddle.metric.Accuracy())
-
         mlflow.paddle.autolog()
-
         with mlflow.start_run() as run:
             model.fit(train_dataset, eval_dataset, batch_size=16, epochs=10)
-
         show_run_data(run.info.run_id)
 
     .. code-block:: text
@@ -603,6 +595,9 @@ def autolog(
         ]
     """
     import paddle
+
     from mlflow.paddle._paddle_autolog import patched_fit
 
-    safe_patch(FLAVOR_NAME, paddle.Model, "fit", patched_fit, manage_run=True)
+    safe_patch(
+        FLAVOR_NAME, paddle.Model, "fit", patched_fit, manage_run=True, extra_tags=extra_tags
+    )
