@@ -873,7 +873,7 @@ class UcModelRegistryStore(BaseRestStore):
                     f"Unable to download model artifacts from source artifact location "
                     f"'{source}' in order to upload them to Unity Catalog. Please ensure "
                     f"the source artifact location exists and that you can download from "
-                    f"it via mlflow.artifacts.download_artifacts()"
+                    f"it via mlflow.artifacts.download_artifacts(). Original error: {e}"
                 ) from e
             try:
                 yield local_model_dir
@@ -894,6 +894,110 @@ class UcModelRegistryStore(BaseRestStore):
             return None
         return mlflow.get_logged_model(model_id)
 
+    def _create_model_version_with_optional_signature_validation(
+        self,
+        name,
+        source,
+        run_id=None,
+        tags=None,
+        run_link=None,
+        description=None,
+        local_model_path=None,
+        model_id: str | None = None,
+        bypass_signature_validation: bool = False,
+        source_workspace_id: str | None = None,
+    ):
+        """
+        Private method to create a new model version from given source and run ID, with optional
+        bypass of signature validation. This bypass is currently only used by the
+        DatabricksWorkspaceModelRegistryRestStore to migrate model versions from the Databricks
+        workspace registry to Unity Catalog via copy_model_version. We do not want to allow
+        normal use of create_model_version to bypass signature validation, so we have this
+        private method.
+
+        Args:
+            name: Registered model name.
+            source: URI indicating the location of the model artifacts.
+            run_id: Run ID from MLflow tracking server that generated the model.
+            tags: A list of :py:class:`mlflow.entities.model_registry.ModelVersionTag`
+                instances associated with this model version.
+            run_link: Link to the run from an MLflow tracking server that generated this model.
+            description: Description of the version.
+            local_model_path: Local path to the MLflow model, if it's already accessible on the
+                local filesystem. Can be used by AbstractStores that upload model version files
+                to the model registry to avoid a redundant download from the source location when
+                logging and registering a model via a single
+                mlflow.<flavor>.log_model(..., registered_model_name) call.
+            model_id: The ID of the model (from an Experiment) that is being promoted to a
+                registered model version, if applicable.
+            bypass_signature_validation: Whether to bypass signature validation.
+            source_workspace_id: The workspace ID of the source run. If not provided,
+                it will be fetched from the run headers.
+
+        Returns:
+            A single object of :py:class:`mlflow.entities.model_registry.ModelVersion`
+            created in the backend.
+        """
+        _require_arg_unspecified(arg_name="run_link", arg_value=run_link)
+        logged_model = self._get_logged_model_from_model_id(model_id)
+        if logged_model:
+            run_id = logged_model.source_run_id
+        headers, run = self._get_run_and_headers(run_id)
+        if source_workspace_id is None:
+            source_workspace_id = self._get_workspace_id(headers)
+        notebook_id = self._get_notebook_id(run)
+        lineage_securable_list = self._get_lineage_input_sources(run)
+        job_id = self._get_job_id(run)
+        job_run_id = self._get_job_run_id(run)
+        extra_headers = None
+        if notebook_id is not None or job_id is not None:
+            entity_list = []
+            lineage_list = None
+            if notebook_id is not None:
+                notebook_entity = Notebook(id=str(notebook_id))
+                entity_list.append(Entity(notebook=notebook_entity))
+            if job_id is not None:
+                job_entity = Job(id=job_id, job_run_id=job_run_id)
+                entity_list.append(Entity(job=job_entity))
+            if lineage_securable_list is not None:
+                lineage_list = [Lineage(source_securables=lineage_securable_list)]
+            lineage_header_info = LineageHeaderInfo(entities=entity_list, lineages=lineage_list)
+            # Base64-encode the header value to ensure it's valid ASCII,
+            # similar to JWT (see https://stackoverflow.com/a/40347926)
+            header_json = message_to_json(lineage_header_info)
+            header_base64 = base64.b64encode(header_json.encode())
+            extra_headers = {_DATABRICKS_LINEAGE_ID_HEADER: header_base64}
+        full_name = get_full_name_from_sc(name, self.spark)
+        with self._local_model_dir(source, local_model_path) as local_model_dir:
+            if not bypass_signature_validation:
+                self._validate_model_signature(local_model_dir)
+            self._download_model_weights_if_not_saved(local_model_dir)
+            feature_deps = get_feature_dependencies(local_model_dir)
+            other_model_deps = get_model_version_dependencies(local_model_dir)
+            req_body = message_to_json(
+                CreateModelVersionRequest(
+                    name=full_name,
+                    source=source,
+                    run_id=run_id,
+                    description=description,
+                    tags=uc_model_version_tag_from_mlflow_tags(tags),
+                    run_tracking_server_id=source_workspace_id,
+                    feature_deps=feature_deps,
+                    model_version_dependencies=other_model_deps,
+                    model_id=model_id,
+                )
+            )
+            model_version = self._call_endpoint(
+                CreateModelVersionRequest, req_body, extra_headers=extra_headers
+            ).model_version
+
+            store = self._get_artifact_repo(model_version, full_name)
+            store.log_artifacts(local_dir=local_model_dir, artifact_path="")
+            finalized_mv = self._finalize_model_version(
+                name=full_name, version=model_version.version
+            )
+            return model_version_from_uc_proto(finalized_mv)
+
     def create_model_version(
         self,
         name,
@@ -928,63 +1032,17 @@ class UcModelRegistryStore(BaseRestStore):
             A single object of :py:class:`mlflow.entities.model_registry.ModelVersion`
             created in the backend.
         """
-        _require_arg_unspecified(arg_name="run_link", arg_value=run_link)
-        logged_model = self._get_logged_model_from_model_id(model_id)
-        if logged_model:
-            run_id = logged_model.source_run_id
-        headers, run = self._get_run_and_headers(run_id)
-        source_workspace_id = self._get_workspace_id(headers)
-        notebook_id = self._get_notebook_id(run)
-        lineage_securable_list = self._get_lineage_input_sources(run)
-        job_id = self._get_job_id(run)
-        job_run_id = self._get_job_run_id(run)
-        extra_headers = None
-        if notebook_id is not None or job_id is not None:
-            entity_list = []
-            lineage_list = None
-            if notebook_id is not None:
-                notebook_entity = Notebook(id=str(notebook_id))
-                entity_list.append(Entity(notebook=notebook_entity))
-            if job_id is not None:
-                job_entity = Job(id=job_id, job_run_id=job_run_id)
-                entity_list.append(Entity(job=job_entity))
-            if lineage_securable_list is not None:
-                lineage_list = [Lineage(source_securables=lineage_securable_list)]
-            lineage_header_info = LineageHeaderInfo(entities=entity_list, lineages=lineage_list)
-            # Base64-encode the header value to ensure it's valid ASCII,
-            # similar to JWT (see https://stackoverflow.com/a/40347926)
-            header_json = message_to_json(lineage_header_info)
-            header_base64 = base64.b64encode(header_json.encode())
-            extra_headers = {_DATABRICKS_LINEAGE_ID_HEADER: header_base64}
-        full_name = get_full_name_from_sc(name, self.spark)
-        with self._local_model_dir(source, local_model_path) as local_model_dir:
-            self._validate_model_signature(local_model_dir)
-            self._download_model_weights_if_not_saved(local_model_dir)
-            feature_deps = get_feature_dependencies(local_model_dir)
-            other_model_deps = get_model_version_dependencies(local_model_dir)
-            req_body = message_to_json(
-                CreateModelVersionRequest(
-                    name=full_name,
-                    source=source,
-                    run_id=run_id,
-                    description=description,
-                    tags=uc_model_version_tag_from_mlflow_tags(tags),
-                    run_tracking_server_id=source_workspace_id,
-                    feature_deps=feature_deps,
-                    model_version_dependencies=other_model_deps,
-                    model_id=model_id,
-                )
-            )
-            model_version = self._call_endpoint(
-                CreateModelVersionRequest, req_body, extra_headers=extra_headers
-            ).model_version
-
-            store = self._get_artifact_repo(model_version, full_name)
-            store.log_artifacts(local_dir=local_model_dir, artifact_path="")
-            finalized_mv = self._finalize_model_version(
-                name=full_name, version=model_version.version
-            )
-            return model_version_from_uc_proto(finalized_mv)
+        return self._create_model_version_with_optional_signature_validation(
+            name=name,
+            source=source,
+            run_id=run_id,
+            tags=tags,
+            run_link=run_link,
+            description=description,
+            local_model_path=local_model_path,
+            model_id=model_id,
+            bypass_signature_validation=False,
+        )
 
     def _get_artifact_repo(self, model_version, model_name=None):
         def base_credential_refresh_def():
@@ -1676,8 +1734,6 @@ class UcModelRegistryStore(BaseRestStore):
             prompt_versions: List of PromptVersion objects to link.
             trace_id: Trace ID to link to each prompt version.
         """
-        super().link_prompts_to_trace(prompt_versions=prompt_versions, trace_id=trace_id)
-
         prompt_version_entries = [
             PromptVersionLinkEntry(name=pv.name, version=str(pv.version)) for pv in prompt_versions
         ]
