@@ -4,7 +4,7 @@ import os
 import posixpath
 import re
 import urllib.parse
-from typing import Union
+from datetime import timezone
 
 from mlflow.entities import FileInfo
 from mlflow.entities.multipart_upload import (
@@ -17,7 +17,7 @@ from mlflow.store.artifact.artifact_repo import ArtifactRepository, MultipartUpl
 from mlflow.utils.credentials import get_default_host_creds
 
 
-def encode_base64(data: Union[str, bytes]) -> str:
+def encode_base64(data: str | bytes) -> str:
     if isinstance(data, str):
         data = data.encode("utf-8")
     encoded = base64.b64encode(data)
@@ -41,8 +41,14 @@ class AzureBlobArtifactRepository(ArtifactRepository, MultipartUploadMixin):
     - DefaultAzureCredential is configured
     """
 
-    def __init__(self, artifact_uri, client=None):
-        super().__init__(artifact_uri)
+    def __init__(
+        self,
+        artifact_uri: str,
+        client=None,
+        tracking_uri: str | None = None,
+        registry_uri: str | None = None,
+    ) -> None:
+        super().__init__(artifact_uri, tracking_uri, registry_uri)
 
         _DEFAULT_TIMEOUT = 600  # 10 minutes
         self.write_timeout = MLFLOW_ARTIFACT_UPLOAD_DOWNLOAD_TIMEOUT.get() or _DEFAULT_TIMEOUT
@@ -232,7 +238,7 @@ class AzureBlobArtifactRepository(ArtifactRepository, MultipartUploadMixin):
             blob_name=dest_path,
             account_key=self.client.credential.account_key,
             permission=BlobSasPermissions(read=True, write=True),
-            expiry=datetime.datetime.utcnow() + datetime.timedelta(hours=1),
+            expiry=datetime.datetime.now(timezone.utc) + datetime.timedelta(hours=1),
         )
         credentials = []
         for i in range(1, num_parts + 1):
